import logging

from carto.auth import APIKeyAuthClient
from carto.exceptions import CartoException
from carto.sql import SQLClient
from django import forms
from django.core.exceptions import ValidationError
from django.utils import six

from .models import CartoDBTable

logger = logging.getLogger(__name__)


class CartoDBTableForm(forms.ModelForm):

    class Meta:
        model = CartoDBTable
        fields = "__all__"

    def clean(self):

        api_key = self.cleaned_data["api_key"]
        domain = self.cleaned_data["domain"]
        table_name = self.cleaned_data["table_name"]
        name_col = self.cleaned_data["name_col"]
        pcode_col = self.cleaned_data["pcode_col"]
        parent_code_col = self.cleaned_data["parent_code_col"]

        auth_client = APIKeyAuthClient(
            api_key=api_key, base_url="https://{}.carto.com/".format(six.text_type(domain))
        )
        sql_client = SQLClient(auth_client)
        try:
            sites = sql_client.send("select * from {} limit 1".format(table_name))
        except CartoException:
            logger.exception("CartoDB exception occured")
            raise ValidationError(f"Couldn't connect to CartoDB table: {table_name}")
        else:
            row = sites["rows"][0]
            if name_col not in row:
                raise ValidationError(
                    "The Name column ({}) is not in table: {}".format(name_col, table_name)
                )
            if pcode_col not in row:
                raise ValidationError(
                    "The PCode column ({}) is not in table: {}".format(pcode_col, table_name)
                )
            if parent_code_col and parent_code_col not in row:
                raise ValidationError(
                    "The Parent Code column ({}) is not in table: {}".format(
                        parent_code_col, table_name
                    )
                )

        return self.cleaned_data
